@@ -1,6 +1,6 @@
 /* mupdate.c -- cyrus murder database master 
  *
- * $Id: mupdate.c,v 1.97 2007/06/08 14:06:43 murch Exp $
+ * $Id: mupdate.c,v 1.99 2007/10/10 15:14:39 murch Exp $
  * Copyright (c) 1998-2003 Carnegie Mellon University.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -87,6 +87,8 @@
 #include "util.h"
 #include "version.h"
 #include "xmalloc.h"
+#include "xstrlcat.h"
+#include "xstrlcpy.h"
 
 /* Sent to clients that we can't accept a connection for. */
 static const char SERVER_UNABLE_STRING[] = "* BYE \"Server Unable\"\r\n";
@@ -251,7 +253,7 @@ static struct conn *conn_new(int fd)
     struct sockaddr_storage localaddr, remoteaddr;
     int r;    
     int haveaddr = 0;
-    int salen;
+    socklen_t salen;
     char hbuf[NI_MAXHOST];
     int niflags;
 
@@ -1074,7 +1076,7 @@ static void dobanner(struct conn *c)
 {
     char slavebuf[4096];
     const char *mechs;
-    unsigned int mechcount;
+    int mechcount;
     int ret;
 
     /* send initial the banner + flush pout */
@@ -1472,6 +1474,7 @@ void cmd_authenticate(struct conn *C,
 		      const char *clientstart)
 {
     int r, sasl_result;
+    const void *val;
 
     r = saslserver(C->saslconn, mech, clientstart, "", "", "",
 		   C->pin, C->pout, &sasl_result, NULL);
@@ -1508,13 +1511,14 @@ void cmd_authenticate(struct conn *C,
     }
 
     /* Successful Authentication */
-    r = sasl_getprop(C->saslconn, SASL_USERNAME, (const void **)&C->userid);
+    r = sasl_getprop(C->saslconn, SASL_USERNAME, &val);
     if(r != SASL_OK) {
 	prot_printf(C->pout, "%s NO \"SASL Error\"\r\n", tag);
 	reset_saslconn(C);
 	return;
     }
 
+    C->userid = (char *) val;
     syslog(LOG_NOTICE, "login: %s %s %s%s %s", C->clienthost, C->userid,
 	   mech, C->tlsconn ? "+TLS" : "", "User logged in");
 
